@@ -16,6 +16,7 @@
 import base64
 
 from mistral_lib.actions import context as lib_ctx
+from mistral_lib import serialization
 from oslo_config import cfg
 from oslo_context import context as oslo_context
 import oslo_messaging as messaging
@@ -26,7 +27,6 @@ from pecan import hooks
 
 from mistral import auth
 from mistral import exceptions as exc
-from mistral import serialization
 from mistral import utils
 
 CONF = cfg.CONF
@@ -132,10 +132,20 @@ def _extract_mistral_auth_params(headers):
     service_catalog = None
 
     if headers.get("X-Target-Auth-Uri"):
+        insecure_header = headers.get('X-Target-Insecure', 'False')
+        if insecure_header == 'False':
+            insecure = False
+        elif insecure_header == 'True':
+            insecure = True
+        else:
+            raise (exc.MistralException(
+                'X-Target-Insecure must be either "True", "False" or not '
+                'provided. The default is "False".'))
+
         params = {
             # TODO(akovi): Target cert not handled yet
             'auth_cacert': None,
-            'insecure': headers.get('X-Target-Insecure', False),
+            'insecure': insecure,
             'auth_token': headers.get('X-Target-Auth-Token'),
             'auth_uri': headers.get('X-Target-Auth-Uri'),
             'tenant': headers.get('X-Target-Project-Id'),
