@@ -75,13 +75,13 @@ def acquire_lock(model, id):
 
 # Workbooks.
 
-def get_workbook(name):
-    return IMPL.get_workbook(name)
+def get_workbook(name, namespace, fields=()):
+    return IMPL.get_workbook(name, namespace=namespace, fields=fields)
 
 
-def load_workbook(name):
+def load_workbook(name, namespace, fields=()):
     """Unlike get_workbook this method is allowed to return None."""
-    return IMPL.load_workbook(name)
+    return IMPL.load_workbook(name, namespace=namespace, fields=fields)
 
 
 def get_workbooks(limit=None, marker=None, sort_keys=None,
@@ -108,8 +108,8 @@ def create_or_update_workbook(name, values):
     return IMPL.create_or_update_workbook(name, values)
 
 
-def delete_workbook(name):
-    IMPL.delete_workbook(name)
+def delete_workbook(name, namespace=None):
+    IMPL.delete_workbook(name, namespace)
 
 
 def delete_workbooks(**kwargs):
@@ -118,17 +118,21 @@ def delete_workbooks(**kwargs):
 
 # Workflow definitions.
 
-def get_workflow_definition(identifier, namespace=''):
-    return IMPL.get_workflow_definition(identifier, namespace=namespace)
+def get_workflow_definition(identifier, namespace='', fields=()):
+    return IMPL.get_workflow_definition(
+        identifier,
+        namespace=namespace,
+        fields=fields
+    )
 
 
-def get_workflow_definition_by_id(id):
-    return IMPL.get_workflow_definition_by_id(id)
+def get_workflow_definition_by_id(id, fields=()):
+    return IMPL.get_workflow_definition_by_id(id, fields=fields)
 
 
-def load_workflow_definition(name, namespace=''):
+def load_workflow_definition(name, namespace='', fields=()):
     """Unlike get_workflow_definition this method is allowed to return None."""
-    return IMPL.load_workflow_definition(name, namespace)
+    return IMPL.load_workflow_definition(name, namespace, fields=fields)
 
 
 def get_workflow_definitions(limit=None, marker=None, sort_keys=None,
@@ -147,8 +151,8 @@ def create_workflow_definition(values):
     return IMPL.create_workflow_definition(values)
 
 
-def update_workflow_definition(identifier, values, namespace):
-    return IMPL.update_workflow_definition(identifier, values, namespace)
+def update_workflow_definition(identifier, values):
+    return IMPL.update_workflow_definition(identifier, values)
 
 
 def create_or_update_workflow_definition(name, values):
@@ -165,17 +169,17 @@ def delete_workflow_definitions(**kwargs):
 
 # Action definitions.
 
-def get_action_definition_by_id(id):
-    return IMPL.get_action_definition_by_id(id)
+def get_action_definition_by_id(id, fields=()):
+    return IMPL.get_action_definition_by_id(id, fields=fields)
 
 
-def get_action_definition(name):
-    return IMPL.get_action_definition(name)
+def get_action_definition(name, fields=()):
+    return IMPL.get_action_definition(name, fields=fields)
 
 
-def load_action_definition(name):
+def load_action_definition(name, fields=()):
     """Unlike get_action_definition this method is allowed to return None."""
-    return IMPL.load_action_definition(name)
+    return IMPL.load_action_definition(name, fields=fields)
 
 
 def get_action_definitions(limit=None, marker=None, sort_keys=None,
@@ -211,13 +215,13 @@ def delete_action_definitions(**kwargs):
 
 # Action executions.
 
-def get_action_execution(id):
-    return IMPL.get_action_execution(id)
+def get_action_execution(id, fields=(), insecure=False):
+    return IMPL.get_action_execution(id, fields=fields, insecure=insecure)
 
 
-def load_action_execution(name):
+def load_action_execution(name, fields=()):
     """Unlike get_action_execution this method is allowed to return None."""
-    return IMPL.load_action_execution(name)
+    return IMPL.load_action_execution(name, fields=fields)
 
 
 def get_action_executions(**kwargs):
@@ -228,8 +232,8 @@ def create_action_execution(values):
     return IMPL.create_action_execution(values)
 
 
-def update_action_execution(id, values):
-    return IMPL.update_action_execution(id, values)
+def update_action_execution(id, values, insecure=False):
+    return IMPL.update_action_execution(id, values, insecure)
 
 
 def create_or_update_action_execution(id, values):
@@ -246,13 +250,13 @@ def delete_action_executions(**kwargs):
 
 # Workflow executions.
 
-def get_workflow_execution(id):
-    return IMPL.get_workflow_execution(id)
+def get_workflow_execution(id, fields=()):
+    return IMPL.get_workflow_execution(id, fields=fields)
 
 
-def load_workflow_execution(name):
+def load_workflow_execution(name, fields=()):
     """Unlike get_workflow_execution this method is allowed to return None."""
-    return IMPL.load_workflow_execution(name)
+    return IMPL.load_workflow_execution(name, fields=fields)
 
 
 def get_workflow_executions(limit=None, marker=None, sort_keys=None,
@@ -292,13 +296,13 @@ def update_workflow_execution_state(**kwargs):
 
 # Tasks executions.
 
-def get_task_execution(id):
-    return IMPL.get_task_execution(id)
+def get_task_execution(id, fields=()):
+    return IMPL.get_task_execution(id, fields=fields)
 
 
-def load_task_execution(id):
+def load_task_execution(id, fields=()):
     """Unlike get_task_execution this method is allowed to return None."""
-    return IMPL.load_task_execution(id)
+    return IMPL.load_task_execution(id, fields=fields)
 
 
 def get_task_executions(limit=None, marker=None, sort_keys=None,
@@ -405,8 +409,7 @@ def get_next_cron_triggers(time):
     return IMPL.get_next_cron_triggers(time)
 
 
-def get_expired_executions(expiration_time, limit=None, columns=(),
-                           session=None):
+def get_expired_executions(expiration_time, limit=None, columns=()):
     return IMPL.get_expired_executions(
         expiration_time,
         limit,
@@ -414,8 +417,12 @@ def get_expired_executions(expiration_time, limit=None, columns=(),
     )
 
 
-def get_superfluous_executions(max_finished_executions, limit=None, columns=(),
-                               session=None):
+def get_running_expired_sync_actions(expiration_time, session=None):
+    return IMPL.get_running_expired_sync_actions(expiration_time)
+
+
+def get_superfluous_executions(max_finished_executions, limit=None,
+                               columns=()):
     return IMPL.get_superfluous_executions(
         max_finished_executions,
         limit,
