@@ -1,5 +1,6 @@
 # Copyright 2016 - Nokia Networks.
 # Copyright 2016 - Brocade Communications Systems, Inc.
+# Copyright 2018 - Extreme Networks, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -14,6 +15,7 @@
 #    limitations under the License.
 
 import abc
+import json
 from oslo_config import cfg
 from oslo_log import log as logging
 from osprofiler import profiler
@@ -27,6 +29,8 @@ from mistral.engine import utils as engine_utils
 from mistral import exceptions as exc
 from mistral import expressions as expr
 from mistral.lang import parser as spec_parser
+from mistral.notifiers import base as notif
+from mistral.notifiers import notification_events as events
 from mistral.services import triggers
 from mistral.services import workflows as wf_service
 from mistral import utils
@@ -62,6 +66,22 @@ class Workflow(object):
         else:
             self.wf_spec = None
 
+    def notify(self, event):
+        publishers = self.wf_ex.params.get('notify')
+
+        if not publishers and not isinstance(publishers, list):
+            return
+
+        notifier = notif.get_notifier(cfg.CONF.notifier.type)
+
+        notifier.notify(
+            self.wf_ex.id,
+            self.wf_ex.to_dict(),
+            event,
+            self.wf_ex.updated_at,
+            publishers
+        )
+
     @profiler.trace('workflow-start')
     def start(self, wf_def, wf_ex_id, input_dict, desc='', params=None):
         """Start workflow.
@@ -101,6 +121,9 @@ class Workflow(object):
 
         self.set_state(states.RUNNING)
 
+        # Publish event as soon as state is set to running.
+        self.notify(events.WORKFLOW_LAUNCHED)
+
         wf_ctrl = wf_base.get_controller(self.wf_ex, self.wf_spec)
 
         dispatcher.dispatch_workflow_commands(
@@ -114,7 +137,6 @@ class Workflow(object):
         :param state: New workflow state.
         :param msg: Additional explaining message.
         """
-
         assert self.wf_ex
 
         if state == states.SUCCESS:
@@ -138,6 +160,9 @@ class Workflow(object):
         # Set the state of this workflow to paused.
         self.set_state(states.PAUSED, state_info=msg)
 
+        # Publish event.
+        self.notify(events.WORKFLOW_PAUSED)
+
         # If workflow execution is a subworkflow,
         # schedule update to the task execution.
         if self.wf_ex.task_execution_id:
@@ -145,8 +170,6 @@ class Workflow(object):
             from mistral.engine import task_handler
             task_handler.schedule_on_action_update(self.wf_ex)
 
-        return
-
     def resume(self, env=None):
         """Resume workflow.
 
@@ -159,6 +182,9 @@ class Workflow(object):
 
         self.set_state(states.RUNNING)
 
+        # Publish event.
+        self.notify(events.WORKFLOW_RESUMED)
+
         wf_ctrl = wf_base.get_controller(self.wf_ex)
 
         # Calculate commands to process next.
@@ -171,6 +197,7 @@ class Workflow(object):
         if self.wf_ex.task_execution_id:
             # Import the task_handler module here to avoid circular reference.
             from mistral.engine import task_handler
+
             task_handler.schedule_on_action_update(self.wf_ex)
 
     def prepare_input(self, input_dict):
@@ -213,8 +240,17 @@ class Workflow(object):
         # Calculate commands to process next.
         cmds = wf_ctrl.rerun_tasks([task_ex], reset=reset)
 
+        if cmds:
+            # Import the task_handler module here to avoid circular reference.
+            from mistral.engine import policies
+
+            policies.RetryPolicy.refresh_runtime_context(task_ex)
+
         self._continue_workflow(cmds)
 
+    def _get_backlog(self):
+        return self.wf_ex.runtime_context.get(dispatcher.BACKLOG_KEY)
+
     def _continue_workflow(self, cmds):
         # When resuming a workflow we need to ignore all 'pause'
         # commands because workflow controller takes tasks that
@@ -231,7 +267,7 @@ class Workflow(object):
             if states.is_completed(t_ex.state) and not t_ex.processed:
                 t_ex.processed = True
 
-        if cmds:
+        if cmds or self._get_backlog():
             dispatcher.dispatch_workflow_commands(self.wf_ex, cmds)
         else:
             self.check_and_complete()
@@ -311,12 +347,16 @@ class Workflow(object):
                 return
 
             self.wf_ex = wf_ex
-            self.wf_ex.state_info = state_info
+            self.wf_ex.state_info = json.dumps(state_info) \
+                if isinstance(state_info, dict) else state_info
 
             wf_trace.info(
                 self.wf_ex,
-                "Workflow '%s' [%s -> %s, msg=%s]"
-                % (self.wf_ex.workflow_name, cur_state, state, state_info)
+                "Workflow '%s' [%s -> %s, msg=%s]" %
+                (self.wf_ex.workflow_name,
+                 cur_state,
+                 state,
+                 self.wf_ex.state_info)
             )
         else:
             msg = ("Can't change workflow execution state from %s to %s. "
@@ -412,6 +452,9 @@ class Workflow(object):
         # Set workflow execution to success until after output is evaluated.
         self.set_state(states.SUCCESS, msg)
 
+        # Publish event.
+        self.notify(events.WORKFLOW_SUCCEEDED)
+
         if self.wf_ex.task_execution_id:
             self._send_result_to_parent_workflow()
 
@@ -457,6 +500,9 @@ class Workflow(object):
 
         self.wf_ex.output = merge_dicts({'result': msg}, output_on_error)
 
+        # Publish event.
+        self.notify(events.WORKFLOW_FAILED)
+
         if self.wf_ex.task_execution_id:
             self._send_result_to_parent_workflow()
 
@@ -475,6 +521,9 @@ class Workflow(object):
 
         self.wf_ex.output = {'result': msg}
 
+        # Publish event.
+        self.notify(events.WORKFLOW_CANCELLED)
+
         if self.wf_ex.task_execution_id:
             self._send_result_to_parent_workflow()
 
