@@ -1,6 +1,7 @@
 # Copyright 2014 - Mirantis, Inc.
 # Copyright 2015 - StackStorm, Inc.
 # Copyright 2017 - Brocade Communications Systems, Inc.
+# Copyright 2018 - Extreme Networks, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -15,6 +16,7 @@
 #    limitations under the License.
 
 from oslo_config import cfg
+from oslo_log import log as logging
 from osprofiler import profiler
 import threading
 
@@ -22,9 +24,12 @@ from mistral import context as auth_ctx
 from mistral.engine import base as eng
 from mistral.event_engine import base as evt_eng
 from mistral.executors import base as exe
+from mistral.notifiers import base as notif
 from mistral.rpc import base
 
 
+LOG = logging.getLogger(__name__)
+
 _ENGINE_CLIENT = None
 _ENGINE_CLIENT_LOCK = threading.Lock()
 
@@ -34,6 +39,9 @@ _EXECUTOR_CLIENT_LOCK = threading.Lock()
 _EVENT_ENGINE_CLIENT = None
 _EVENT_ENGINE_CLIENT_LOCK = threading.Lock()
 
+_NOTIFIER_CLIENT = None
+_NOTIFIER_CLIENT_LOCK = threading.Lock()
+
 
 def cleanup():
     """Clean all the RPC clients.
@@ -46,17 +54,19 @@ def cleanup():
     global _ENGINE_CLIENT
     global _EXECUTOR_CLIENT
     global _EVENT_ENGINE_CLIENT
+    global _NOTIFIER_CLIENT
 
     _ENGINE_CLIENT = None
     _EXECUTOR_CLIENT = None
     _EVENT_ENGINE_CLIENT = None
+    _NOTIFIER_CLIENT = None
 
     base.cleanup()
 
 
 def get_engine_client():
     global _ENGINE_CLIENT
-    global _EVENT_ENGINE_CLIENT_LOCK
+    global _ENGINE_CLIENT_LOCK
 
     with _ENGINE_CLIENT_LOCK:
         if not _ENGINE_CLIENT:
@@ -87,6 +97,17 @@ def get_event_engine_client():
     return _EVENT_ENGINE_CLIENT
 
 
+def get_notifier_client():
+    global _NOTIFIER_CLIENT
+    global _NOTIFIER_CLIENT_LOCK
+
+    with _NOTIFIER_CLIENT_LOCK:
+        if not _NOTIFIER_CLIENT:
+            _NOTIFIER_CLIENT = NotifierClient(cfg.CONF.notifier)
+
+    return _NOTIFIER_CLIENT
+
+
 class EngineClient(eng.Engine):
     """RPC Engine client."""
 
@@ -103,7 +124,6 @@ class EngineClient(eng.Engine):
         """Starts workflow sending a request to engine over RPC.
 
         :param wf_identifier: Workflow identifier.
-        :param wf_namespace Workflow namespace.
         :param wf_namespace: Workflow namespace.
         :param wf_input: Workflow input data as a dictionary.
         :param wf_ex_id: Workflow execution id. If passed, it will be set
@@ -150,8 +170,8 @@ class EngineClient(eng.Engine):
         """Conveys action result to Mistral Engine.
 
         This method should be used by clients of Mistral Engine to update
-        state of a action execution once action has executed. One of the
-        clients of this method is Mistral REST API server that receives
+        the state of an action execution once action has executed. One of
+        the clients of this method is Mistral REST API server that receives
         action result from the outside action handlers.
 
         Note: calling this method serves an event notifying Mistral that
@@ -186,7 +206,7 @@ class EngineClient(eng.Engine):
         """Conveys update of action state to Mistral Engine.
 
         This method should be used by clients of Mistral Engine to update
-        state of a action execution once action has executed.
+        the state of an action execution once action has executed.
 
         Note: calling this method serves an event notifying Mistral that it
         may need to change the state of the parent task and workflow. Use
@@ -302,6 +322,18 @@ class EngineClient(eng.Engine):
             wf_ex_id=wf_ex_id
         )
 
+    @base.wrap_messaging_exception
+    def report_running_actions(self, action_ex_ids):
+        """Receives action execution heartbeats.
+
+        :param action_ex_ids: Action execution ids.
+        """
+        return self._client.async_call(
+            auth_ctx.ctx(),
+            'report_running_actions',
+            action_ex_ids=action_ex_ids
+        )
+
 
 class ExecutorClient(exe.Executor):
     """RPC Executor client."""
@@ -381,3 +413,25 @@ class EventEngineClient(evt_eng.EventEngine):
             'update_event_trigger',
             trigger=trigger,
         )
+
+
+class NotifierClient(notif.Notifier):
+    """RPC Notifier client."""
+
+    def __init__(self, rpc_conf_dict):
+        """Constructs an RPC client for the Notifier service."""
+        self._client = base.get_rpc_client_driver()(rpc_conf_dict)
+
+    def notify(self, ex_id, data, event, timestamp, publishers):
+        try:
+            return self._client.async_call(
+                auth_ctx.ctx(),
+                'notify',
+                ex_id=ex_id,
+                data=data,
+                event=event,
+                timestamp=timestamp,
+                publishers=publishers
+            )
+        except Exception:
+            LOG.exception('Unable to send notification.')
